use Test::More;
use Data::Printer;
use_ok( Pg::BulkLoad );

=pod

If you want to test this module look at the test folder.
This just tests that Perl can load the module. 
Proper testing requires a postgres server.

=cut

done_testing();

